@@ -3,25 +3,38 @@
 
 """Tests for the Cython magics extension."""
 
+from __future__ import absolute_import
+
 import os
 import sys
+from contextlib import contextmanager
+from Cython.Build import IpythonMagic
+from Cython.TestUtils import CythonTest
 
 try:
-    from IPython.testing.globalipapp import get_ipython
+    import IPython.testing.globalipapp
     from IPython.utils import py3compat
-except:
-    __test__ = False
+except ImportError:
+    # Disable tests and fake helpers for initialisation below.
+    class _py3compat(object):
+        def str_to_unicode(self, s):
+            return s
+
+    py3compat = _py3compat()
+
+    def skip_if_not_installed(_):
+        return None
+else:
+    def skip_if_not_installed(c):
+        return c
 
 try:
-    # disable IPython history thread to avoid having to clean it up
+    # disable IPython history thread before it gets started to avoid having to clean it up
     from IPython.core.history import HistoryManager
     HistoryManager.enabled = False
 except ImportError:
     pass
 
-from Cython.TestUtils import CythonTest
-
-ip = get_ipython()
 code = py3compat.str_to_unicode("""\
 def f(x):
     return 2*x
@@ -61,19 +74,27 @@ else:
         return _skip_win32
 
 
+@skip_if_not_installed
 class TestIPythonMagic(CythonTest):
 
+    @classmethod
+    def setUpClass(cls):
+        CythonTest.setUpClass()
+        cls._ip = IPython.testing.globalipapp.get_ipython()
+
     def setUp(self):
         CythonTest.setUp(self)
-        ip.extension_manager.load_extension('cython')
+        self._ip.extension_manager.load_extension('cython')
 
     def test_cython_inline(self):
+        ip = self._ip
         ip.ex('a=10; b=20')
         result = ip.run_cell_magic('cython_inline', '', 'return a+b')
         self.assertEqual(result, 30)
 
     @skip_win32('Skip on Windows')
     def test_cython_pyximport(self):
+        ip = self._ip
         module_name = '_test_cython_pyximport'
         ip.run_cell_magic('cython_pyximport', module_name, code)
         ip.ex('g = f(10)')
@@ -87,12 +108,14 @@ class TestIPythonMagic(CythonTest):
             pass
 
     def test_cython(self):
+        ip = self._ip
         ip.run_cell_magic('cython', '', code)
         ip.ex('g = f(10)')
         self.assertEqual(ip.user_ns['g'], 20.0)
 
     def test_cython_name(self):
         # The Cython module named 'mymodule' defines the function f.
+        ip = self._ip
         ip.run_cell_magic('cython', '--name=mymodule', code)
         # This module can now be imported in the interactive namespace.
         ip.ex('import mymodule; g = mymodule.f(10)')
@@ -100,6 +123,7 @@ class TestIPythonMagic(CythonTest):
 
     def test_cython_language_level(self):
         # The Cython cell defines the functions f() and call().
+        ip = self._ip
         ip.run_cell_magic('cython', '', cython3_code)
         ip.ex('g = f(10); h = call(10)')
         if sys.version_info[0] < 3:
@@ -111,6 +135,7 @@ class TestIPythonMagic(CythonTest):
 
     def test_cython3(self):
         # The Cython cell defines the functions f() and call().
+        ip = self._ip
         ip.run_cell_magic('cython', '-3', cython3_code)
         ip.ex('g = f(10); h = call(10)')
         self.assertEqual(ip.user_ns['g'], 2.0 / 10.0)
@@ -118,13 +143,16 @@ class TestIPythonMagic(CythonTest):
 
     def test_cython2(self):
         # The Cython cell defines the functions f() and call().
+        ip = self._ip
         ip.run_cell_magic('cython', '-2', cython3_code)
         ip.ex('g = f(10); h = call(10)')
         self.assertEqual(ip.user_ns['g'], 2 // 10)
         self.assertEqual(ip.user_ns['h'], 2 // 10)
 
+    @skip_win32('Skip on Windows')
     def test_cython3_pgo(self):
         # The Cython cell defines the functions f() and call().
+        ip = self._ip
         ip.run_cell_magic('cython', '-3 --pgo', pgo_cython3_code)
         ip.ex('g = f(10); h = call(10); main()')
         self.assertEqual(ip.user_ns['g'], 2.0 / 10.0)
@@ -132,6 +160,7 @@ class TestIPythonMagic(CythonTest):
 
     @skip_win32('Skip on Windows')
     def test_extlibs(self):
+        ip = self._ip
         code = py3compat.str_to_unicode("""
 from libc.math cimport sin
 x = sin(0.0)
@@ -139,3 +168,45 @@ x = sin(0.0)
         ip.user_ns['x'] = 1
         ip.run_cell_magic('cython', '-l m', code)
         self.assertEqual(ip.user_ns['x'], 0)
+
+
+    def test_cython_verbose(self):
+        ip = self._ip
+        ip.run_cell_magic('cython', '--verbose', code)
+        ip.ex('g = f(10)')
+        self.assertEqual(ip.user_ns['g'], 20.0)
+
+    def test_cython_verbose_thresholds(self):
+        @contextmanager
+        def mock_distutils():
+            class MockLog:
+                DEBUG = 1
+                INFO = 2
+                thresholds = [INFO]
+
+                def set_threshold(self, val):
+                    self.thresholds.append(val)
+                    return self.thresholds[-2]
+
+
+            new_log = MockLog()
+            old_log = IpythonMagic.distutils.log
+            try:
+                IpythonMagic.distutils.log = new_log
+                yield new_log
+            finally:
+                IpythonMagic.distutils.log = old_log
+
+        ip = self._ip
+        with mock_distutils() as verbose_log:
+            ip.run_cell_magic('cython', '--verbose', code)
+            ip.ex('g = f(10)')
+        self.assertEqual(ip.user_ns['g'], 20.0)
+        self.assertEquals([verbose_log.INFO, verbose_log.DEBUG, verbose_log.INFO],
+                          verbose_log.thresholds)
+
+        with mock_distutils() as normal_log:
+            ip.run_cell_magic('cython', '', code)
+            ip.ex('g = f(10)')
+        self.assertEqual(ip.user_ns['g'], 20.0)
+        self.assertEquals([normal_log.INFO], normal_log.thresholds)
